from distutils.core import setup

setup(
  name = 'bobot',
  packages = ['bobot'],
  version = '0.1.4',
  description = 'Telegram Bot API Wrapper',
  author = 'Trdat Mkrtchyan',
  author_email = 'wombtormb@gmail.com',
  url = 'https://github.com/zefirka/bobot',
  download_url = 'https://github.com/zefirka/bobot/archive/v0.1.4.tar.gz',
  keywords = ['telegram', 'bot', 'api'],
  classifiers = []
)




